@@ -832,6 +832,14 @@ pub trait AsArray: private::Sealed {
         self.as_list_opt().expect("list array")
     }
 
+    /// Downcast this to a [`GenericListViewArray`] returning `None` if not possible
+    fn as_list_view_opt<O: OffsetSizeTrait>(&self) -> Option<&GenericListViewArray<O>>;
+
+    /// Downcast this to a [`GenericListViewArray`] panicking if not possible
+    fn as_list_view<O: OffsetSizeTrait>(&self) -> &GenericListViewArray<O> {
+        self.as_list_view_opt().expect("list view array")
+    }
+
     /// Downcast this to a [`FixedSizeBinaryArray`] returning `None` if not possible
     fn as_fixed_size_binary_opt(&self) -> Option<&FixedSizeBinaryArray>;
 
@@ -905,6 +913,10 @@ impl AsArray for dyn Array + '_ {
         self.as_any().downcast_ref()
     }
 
+    fn as_list_view_opt<O: OffsetSizeTrait>(&self) -> Option<&GenericListViewArray<O>> {
+        self.as_any().downcast_ref()
+    }
+
     fn as_fixed_size_binary_opt(&self) -> Option<&FixedSizeBinaryArray> {
         self.as_any().downcast_ref()
     }
@@ -960,6 +972,10 @@ impl AsArray for ArrayRef {
         self.as_ref().as_list_opt()
     }
 
+    fn as_list_view_opt<O: OffsetSizeTrait>(&self) -> Option<&GenericListViewArray<O>> {
+        self.as_ref().as_list_view_opt()
+    }
+
     fn as_fixed_size_binary_opt(&self) -> Option<&FixedSizeBinaryArray> {
         self.as_ref().as_fixed_size_binary_opt()
     }
