@@ -16,7 +16,8 @@
 // under the License.
 
 use std::collections::HashMap;
-use std::ops::Range;
+
+use bytes::Bytes;
 
 use crate::basic::Encoding;
 use crate::data_type::DataType;
@@ -26,57 +27,20 @@ use crate::encodings::{
 };
 use crate::errors::{ParquetError, Result};
 use crate::schema::types::ColumnDescPtr;
-use crate::util::{
-    bit_util::{num_required_bits, BitReader},
-    memory::ByteBufferPtr,
-};
+use crate::util::bit_util::{num_required_bits, BitReader};
 
-/// A slice of levels buffer data that is written to by a [`ColumnLevelDecoder`]
-pub trait LevelsBufferSlice {
-    /// Returns the capacity of this slice or `usize::MAX` if no limit
-    fn capacity(&self) -> usize;
-
-    /// Count the number of levels in `range` not equal to `max_level`
-    fn count_nulls(&self, range: Range<usize>, max_level: i16) -> usize;
-}
-
-impl LevelsBufferSlice for [i16] {
-    fn capacity(&self) -> usize {
-        self.len()
-    }
-
-    fn count_nulls(&self, range: Range<usize>, max_level: i16) -> usize {
-        self[range].iter().filter(|i| **i != max_level).count()
-    }
-}
-
-/// A slice of values buffer data that is written to by a [`ColumnValueDecoder`]
-pub trait ValuesBufferSlice {
-    /// Returns the capacity of this slice or `usize::MAX` if no limit
-    fn capacity(&self) -> usize;
-}
-
-impl<T> ValuesBufferSlice for [T] {
-    fn capacity(&self) -> usize {
-        self.len()
-    }
-}
-
-/// Decodes level data to a [`LevelsBufferSlice`]
+/// Decodes level data
 pub trait ColumnLevelDecoder {
-    type Slice: LevelsBufferSlice + ?Sized;
+    type Buffer;
 
     /// Set data for this [`ColumnLevelDecoder`]
-    fn set_data(&mut self, encoding: Encoding, data: ByteBufferPtr);
+    fn set_data(&mut self, encoding: Encoding, data: Bytes);
 }
 
 pub trait RepetitionLevelDecoder: ColumnLevelDecoder {
-    /// Read up to `max_records` of repetition level data into `out[range]` returning the number
+    /// Read up to `max_records` of repetition level data into `out` returning the number
     /// of complete records and levels read
     ///
-    /// `range` is provided by the caller to allow for types such as default-initialized `[T]`
-    /// that only track capacity and not length
-    ///
     /// A record only ends when the data contains a subsequent repetition level of 0,
     /// it is therefore left to the caller to delimit the final record in a column
     ///
@@ -85,9 +49,9 @@ pub trait RepetitionLevelDecoder: ColumnLevelDecoder {
     /// Implementations may panic if `range` overlaps with already written data
     fn read_rep_levels(
         &mut self,
-        out: &mut Self::Slice,
-        range: Range<usize>,
-        max_records: usize,
+        out: &mut Self::Buffer,
+        num_records: usize,
+        num_levels: usize,
     ) -> Result<(usize, usize)>;
 
     /// Skips over up to `num_levels` repetition levels corresponding to `num_records` records,
@@ -104,27 +68,28 @@ pub trait RepetitionLevelDecoder: ColumnLevelDecoder {
 }
 
 pub trait DefinitionLevelDecoder: ColumnLevelDecoder {
-    /// Read definition level data into `out[range]` returning the number of levels read
+    /// Read up to `num_levels` definition levels into `out`
     ///
-    /// `range` is provided by the caller to allow for types such as default-initialized `[T]`
-    /// that only track capacity and not length
+    /// Returns the number of values skipped, and the number of levels skipped
     ///
     /// # Panics
     ///
     /// Implementations may panic if `range` overlaps with already written data
-    ///
-    // TODO: Should this return the number of nulls
-    fn read_def_levels(&mut self, out: &mut Self::Slice, range: Range<usize>) -> Result<usize>;
+    fn read_def_levels(
+        &mut self,
+        out: &mut Self::Buffer,
+        num_levels: usize,
+    ) -> Result<(usize, usize)>;
 
     /// Skips over `num_levels` definition levels
     ///
     /// Returns the number of values skipped, and the number of levels skipped
-    fn skip_def_levels(&mut self, num_levels: usize, max_def_level: i16) -> Result<(usize, usize)>;
+    fn skip_def_levels(&mut self, num_levels: usize) -> Result<(usize, usize)>;
 }
 
-/// Decodes value data to a [`ValuesBufferSlice`]
+/// Decodes value data
 pub trait ColumnValueDecoder {
-    type Slice: ValuesBufferSlice + ?Sized;
+    type Buffer;
 
     /// Create a new [`ColumnValueDecoder`]
     fn new(col: &ColumnDescPtr) -> Self;
@@ -132,7 +97,7 @@ pub trait ColumnValueDecoder {
     /// Set the current dictionary page
     fn set_dict(
         &mut self,
-        buf: ByteBufferPtr,
+        buf: Bytes,
         num_values: u32,
         encoding: Encoding,
         is_sorted: bool,
@@ -152,21 +117,18 @@ pub trait ColumnValueDecoder {
     fn set_data(
         &mut self,
         encoding: Encoding,
-        data: ByteBufferPtr,
+        data: Bytes,
         num_levels: usize,
         num_values: Option<usize>,
     ) -> Result<()>;
 
-    /// Read values data into `out[range]` returning the number of values read
-    ///
-    /// `range` is provided by the caller to allow for types such as default-initialized `[T]`
-    /// that only track capacity and not length
+    /// Read up to `num_values` values into `out`
     ///
     /// # Panics
     ///
     /// Implementations may panic if `range` overlaps with already written data
     ///
-    fn read(&mut self, out: &mut Self::Slice, range: Range<usize>) -> Result<usize>;
+    fn read(&mut self, out: &mut Self::Buffer, num_values: usize) -> Result<usize>;
 
     /// Skips over `num_values` values
     ///
@@ -185,7 +147,7 @@ pub struct ColumnValueDecoderImpl<T: DataType> {
 }
 
 impl<T: DataType> ColumnValueDecoder for ColumnValueDecoderImpl<T> {
-    type Slice = [T::T];
+    type Buffer = Vec<T::T>;
 
     fn new(descr: &ColumnDescPtr) -> Self {
         Self {
@@ -197,7 +159,7 @@ impl<T: DataType> ColumnValueDecoder for ColumnValueDecoderImpl<T> {
 
     fn set_dict(
         &mut self,
-        buf: ByteBufferPtr,
+        buf: Bytes,
         num_values: u32,
         mut encoding: Encoding,
         _is_sorted: bool,
@@ -229,7 +191,7 @@ impl<T: DataType> ColumnValueDecoder for ColumnValueDecoderImpl<T> {
     fn set_data(
         &mut self,
         mut encoding: Encoding,
-        data: ByteBufferPtr,
+        data: Bytes,
         num_levels: usize,
         num_values: Option<usize>,
     ) -> Result<()> {
@@ -259,7 +221,7 @@ impl<T: DataType> ColumnValueDecoder for ColumnValueDecoderImpl<T> {
         Ok(())
     }
 
-    fn read(&mut self, out: &mut Self::Slice, range: Range<usize>) -> Result<usize> {
+    fn read(&mut self, out: &mut Self::Buffer, num_values: usize) -> Result<usize> {
         let encoding = self
             .current_encoding
             .expect("current_encoding should be set");
@@ -269,7 +231,12 @@ impl<T: DataType> ColumnValueDecoder for ColumnValueDecoderImpl<T> {
             .get_mut(&encoding)
             .unwrap_or_else(|| panic!("decoder for encoding {encoding} should be set"));
 
-        current_decoder.get(&mut out[range])
+        // TODO: Push vec into decoder (#5177)
+        let start = out.len();
+        out.resize(start + num_values, T::T::default());
+        let read = current_decoder.get(&mut out[start..])?;
+        out.truncate(start + read);
+        Ok(read)
     }
 
     fn skip_values(&mut self, num_values: usize) -> Result<usize> {
@@ -294,13 +261,14 @@ enum LevelDecoder {
 }
 
 impl LevelDecoder {
-    fn new(encoding: Encoding, data: ByteBufferPtr, bit_width: u8) -> Self {
+    fn new(encoding: Encoding, data: Bytes, bit_width: u8) -> Self {
         match encoding {
             Encoding::RLE => {
                 let mut decoder = RleDecoder::new(bit_width);
                 decoder.set_data(data);
                 Self::Rle(decoder)
             }
+            #[allow(deprecated)]
             Encoding::BIT_PACKED => Self::Packed(BitReader::new(data), bit_width),
             _ => unreachable!("invalid level encoding: {}", encoding),
         }
@@ -320,6 +288,7 @@ impl LevelDecoder {
 pub struct DefinitionLevelDecoderImpl {
     decoder: Option<LevelDecoder>,
     bit_width: u8,
+    max_level: i16,
 }
 
 impl DefinitionLevelDecoderImpl {
@@ -328,24 +297,37 @@ impl DefinitionLevelDecoderImpl {
         Self {
             decoder: None,
             bit_width,
+            max_level,
         }
     }
 }
 
 impl ColumnLevelDecoder for DefinitionLevelDecoderImpl {
-    type Slice = [i16];
+    type Buffer = Vec<i16>;
 
-    fn set_data(&mut self, encoding: Encoding, data: ByteBufferPtr) {
+    fn set_data(&mut self, encoding: Encoding, data: Bytes) {
         self.decoder = Some(LevelDecoder::new(encoding, data, self.bit_width))
     }
 }
 
 impl DefinitionLevelDecoder for DefinitionLevelDecoderImpl {
-    fn read_def_levels(&mut self, out: &mut Self::Slice, range: Range<usize>) -> Result<usize> {
-        self.decoder.as_mut().unwrap().read(&mut out[range])
+    fn read_def_levels(
+        &mut self,
+        out: &mut Self::Buffer,
+        num_levels: usize,
+    ) -> Result<(usize, usize)> {
+        // TODO: Push vec into decoder (#5177)
+        let start = out.len();
+        out.resize(start + num_levels, 0);
+        let levels_read = self.decoder.as_mut().unwrap().read(&mut out[start..])?;
+        out.truncate(start + levels_read);
+
+        let iter = out.iter().skip(start);
+        let values_read = iter.filter(|x| **x == self.max_level).count();
+        Ok((values_read, levels_read))
     }
 
-    fn skip_def_levels(&mut self, num_levels: usize, max_def_level: i16) -> Result<(usize, usize)> {
+    fn skip_def_levels(&mut self, num_levels: usize) -> Result<(usize, usize)> {
         let mut level_skip = 0;
         let mut value_skip = 0;
         let mut buf: Vec<i16> = vec![];
@@ -354,14 +336,14 @@ impl DefinitionLevelDecoder for DefinitionLevelDecoderImpl {
 
             let to_read = remaining_levels.min(SKIP_BUFFER_SIZE);
             buf.resize(to_read, 0);
-            let read = self.read_def_levels(&mut buf, 0..to_read)?;
-            if read == 0 {
+            let (values_read, levels_read) = self.read_def_levels(&mut buf, to_read)?;
+            if levels_read == 0 {
                 // Reached end of page
                 break;
             }
 
-            level_skip += read;
-            value_skip += buf[..read].iter().filter(|x| **x == max_def_level).count();
+            level_skip += levels_read;
+            value_skip += values_read;
         }
 
         Ok((value_skip, level_skip))
@@ -424,9 +406,9 @@ impl RepetitionLevelDecoderImpl {
 }
 
 impl ColumnLevelDecoder for RepetitionLevelDecoderImpl {
-    type Slice = [i16];
+    type Buffer = Vec<i16>;
 
-    fn set_data(&mut self, encoding: Encoding, data: ByteBufferPtr) {
+    fn set_data(&mut self, encoding: Encoding, data: Bytes) {
         self.decoder = Some(LevelDecoder::new(encoding, data, self.bit_width));
         self.buffer_len = 0;
         self.buffer_offset = 0;
@@ -436,16 +418,14 @@ impl ColumnLevelDecoder for RepetitionLevelDecoderImpl {
 impl RepetitionLevelDecoder for RepetitionLevelDecoderImpl {
     fn read_rep_levels(
         &mut self,
-        out: &mut Self::Slice,
-        range: Range<usize>,
-        max_records: usize,
+        out: &mut Self::Buffer,
+        num_records: usize,
+        num_levels: usize,
     ) -> Result<(usize, usize)> {
-        let output = &mut out[range];
-        let max_levels = output.len();
         let mut total_records_read = 0;
         let mut total_levels_read = 0;
 
-        while total_records_read < max_records && total_levels_read < max_levels {
+        while total_records_read < num_records && total_levels_read < num_levels {
             if self.buffer_len == self.buffer_offset {
                 self.fill_buf()?;
                 if self.buffer_len == 0 {
@@ -454,11 +434,11 @@ impl RepetitionLevelDecoder for RepetitionLevelDecoderImpl {
             }
 
             let (partial, records_read, levels_read) = self.count_records(
-                max_records - total_records_read,
-                max_levels - total_levels_read,
+                num_records - total_records_read,
+                num_levels - total_levels_read,
             );
 
-            output[total_levels_read..total_levels_read + levels_read].copy_from_slice(
+            out.extend_from_slice(
                 &self.buffer[self.buffer_offset..self.buffer_offset + levels_read],
             );
 
@@ -511,7 +491,7 @@ mod tests {
         let mut encoder = RleEncoder::new(1, 1024);
         encoder.put(0);
         (0..3).for_each(|_| encoder.put(1));
-        let data = ByteBufferPtr::new(encoder.consume());
+        let data = Bytes::from(encoder.consume());
 
         let mut decoder = RepetitionLevelDecoderImpl::new(1);
         decoder.set_data(Encoding::RLE, data.clone());
@@ -537,7 +517,7 @@ mod tests {
             for v in &encoded {
                 encoder.put(*v as _)
             }
-            let data = ByteBufferPtr::new(encoder.consume());
+            let data = Bytes::from(encoder.consume());
 
             let mut decoder = RepetitionLevelDecoderImpl::new(5);
             decoder.set_data(Encoding::RLE, data);
@@ -551,13 +531,13 @@ mod tests {
                 let (records_read, levels_read) = if skip {
                     decoder.skip_rep_levels(records, remaining_levels).unwrap()
                 } else {
-                    let mut decoded = vec![0; remaining_levels];
+                    let mut decoded = Vec::new();
                     let (records_read, levels_read) = decoder
-                        .read_rep_levels(&mut decoded, 0..remaining_levels, records)
+                        .read_rep_levels(&mut decoded, records, remaining_levels)
                         .unwrap();
 
                     assert_eq!(
-                        decoded[..levels_read],
+                        decoded,
                         encoded[encoded.len() - remaining_levels..][..levels_read]
                     );
                     (records_read, levels_read)
